<?php

include_once(realpath(dirname(__FILE__)) . "/Shopbotexportservice.class.php");

/**
 * Classe abstraite représentant un comparateur de prix (à dériver pour chaque comparateur) 
 */
abstract class Comparateur {

    /**
     * Le nom du coomparateur
     * @var string 
     */
    private $nom;

    /**
     * Le chemin absolu de l'image du comparateur
     * @var string 
     */
    private $absolutePathImg;

    /**
     * Le chemin absolu de l'image du comparateur
     * @var string 
     */
    private $relativePathImg;

    /**
     * Le nom de l'image 
     */
    private $nameImg;

    /**
     * Le chemin du comparateur
     * @var string 
     */
    private $rootPath;

    /**
     * L'id du comparateur dans la base de données
     * @var int 
     */
    private $idComparateur;

    /**
     * L'etat du comparateur (activé = true, désactivé = false)
     * @var bool 
     */
    private $etat = 0;

    /**
     * La connexion à la base de données
     * @var handle 
     */
    public $cnx;

    /**
     * Les produits du comparateur et leurs etats
     * @var array 
     */
    public $produits = array();

    /**
     * Tableau d'instances des comparateurs
     * @var type 
     */
    private static $instances = array();

    /**
     * Constructeur par défaut d'un comparateur 
     */
    private function __construct() {
        $this->initCnx();
        $this->nom = get_called_class();
        $this->initRootPath();
        $this->initPathImage();
        $this->initIdComparateur();
        $this->initEtat();
        $this->initProduits();
    }

    /**
     * Accesseur du Singleton Comparateur
     * @param string $comparateur : le nom du comparateur voulu
     * @return Comparateur
     */
    public static function getInstance($comparateur) {
        if (!isset(self::$instances[$comparateur])) {
            $comparateur2 = ucfirst($comparateur);
            include_once(realpath(dirname(__FILE__)) . "/comparateurs/" . strtolower($comparateur) . "/" . ucfirst(strtolower($comparateur)) . ".class.php");
            self::$instances[$comparateur] = new $comparateur2();
        }
        return self::$instances[$comparateur];
    }

    /**
     * Initialise les produits et leurs informations liés au comparateur 
     */
    private function initProduits() {
        $result = mysql_query("SELECT * 
                               FROM " . Shopbotexportservice::$tableCompProduit . " 
                               WHERE idComp = '" . $this->idComparateur . "'", $this->cnx);
        if ($result && mysql_num_rows($result) > 0) {
            while ($row = mysql_fetch_assoc($result)) {
                array_push($this->produits, $row);
            }
        }
    }

    /**
     * Initialise l'id du comparateur 
     */
    private function initIdComparateur() {
        $query = "SELECT idComp FROM " . Shopbotexportservice::$tableComparateur . " WHERE nom = '" . ucfirst(strtolower($this->nom)) . "'";
        $result = mysql_query($query, $this->cnx);
        if ($result && mysql_num_rows($result) > 0 && $row = mysql_fetch_object($result)) {
            $this->idComparateur = $row->idComp;
        } else {
            $this->idComparateur = $this->createComparateur();
        }
    }

    /**
     * Créer le comparateur en base
     * @return int 
     */
    private function createComparateur() {
        $query = "INSERT INTO " . Shopbotexportservice::$tableComparateur . " VALUES('', '" . $this->nom . "', '" . $this->pathImg . "', '0');";
        mysql_query($query, $this->cnx);
        return mysql_insert_id($this->cnx);
    }

    /**
     * Initialise l'etat du comparateur 
     */
    private function initEtat() {
        $query = "SELECT etat FROM " . Shopbotexportservice::$tableComparateur . " WHERE idComp = '" . $this->idComparateur . "'";
        $result = mysql_query($query, $this->cnx);
        if ($result && mysql_num_rows($result) > 0 && $row = mysql_fetch_object($result)) {
            $this->etat = $row->etat;
        }
    }

    /**
     * Retourne l'id du comparateur
     * @return int : l'id du comparateur 
     */
    public function getIdComparateur() {
        return $this->idComparateur;
    }

    /**
     * Initilialise l'etat du plugin
     * @param bool $etat 
     */
    public function setEtat($etat) {
        if (is_bool($etat)) {
            ($etat == true) ? $etat = 1 : $etat = 0;
            $query = "UPDATE " . Shopbotexportservice::$tableComparateur . " SET etat = '" . $etat . "' WHERE idComp = '" . $this->idComparateur . "'";
            mysql_query($query, $this->cnx);
            $this->etat = $etat;
        }
    }

    /**
     * Donne l'etat du comparateur 
     * @return bool : l'état du plugin
     */
    public function getEtat() {
        return $this->etat;
    }

    /**
     *  Initialise le chemin par défaut du comparateur sur le serveur 
     */
    private function initRootPath() {
        $this->rootPath = realpath(dirname(__FILE__)) . "/comparateurs/" . strtolower($this->nom) . "/";
    }

    /**
     * Donne le chemin par défaut du comparateur sur le serveur 
     * @return string : le chemin par défaut du comparateur sur le serveur 
     */
    public function getRootPath() {
        return $this->rootPath;
    }

    /**
     * Donne le nom du comparateur
     * @return string
     */
    public function getNom() {
        return $this->nom;
    }

    /**
     * Initialise le chemin de l'image du comparateur (.png, .jpg, .jpeg et .gif autorisés)
     */
    private function initPathImage() {
        if (file_exists($this->rootPath . strtolower($this->nom) . ".png")) {
            $this->absolutePathImg = $this->rootPath . strtolower($this->nom) . ".png";
            $this->relativePathImg = "comparateurs/" . strtolower($this->nom) . "/" . strtolower($this->nom) . ".png";
            $this->nameImg = $this->nom . ".png";
        } else if (file_exists($this->rootPath . strtolower($this->nom) . ".jpg")) {
            $this->absolutePathImg = $this->rootPath . strtolower($this->nom) . ".jpg";
            $this->relativePathImg = "comparateurs/" . strtolower($this->nom) . "/" . strtolower($this->nom) . ".jpg";
            $this->nameImg = $this->nom . ".jpg";
        } else if (file_exists($this->rootPath . strtolower($this->nom) . ".jpeg")) {
            $this->absolutePathImg = $this->rootPath . strtolower($this->nom) . ".jpeg";
            $this->relativePathImg = "comparateurs/" . strtolower($this->nom) . "/" . strtolower($this->nom) . ".jpeg";
            $this->nameImg = $this->nom . ".jpeg";
        } else if (file_exists($this->rootPath . strtolower($this->nom) . ".gif")) {
            $this->absolutePathImgmg = $this->rootPath . strtolower($this->nom) . ".gif";
            $this->relativePathImg = "comparateurs/" . strtolower($this->nom) . "/" . strtolower($this->nom) . ".gif";
            $this->nameImg = $this->nom . ".gif";
        } else {
            $this->absolutePathImg = $this->rootPath . "../images/empty.png";
            $this->relativePathImg = "images/empty.png";
            $this->nameImg = "empty.png";
        }
    }

    /**
     * Donne le chemin absolu de l'image du comparateur
     * @return string: le chemin de l'image du comparateur 
     */
    public function getAbsolutePathImg() {
        return $this->absolutePathImg;
    }

    /**
     * Donne le chemin relatif de l'image du comparateur
     * @return string: le chemin de l'image du comparateur 
     */
    public function getRelativePathImg() {
        return $this->relativePathImg;
    }

    /**
     * Donne le nom de l'image du comparateur
     * @return string: le chemin de l'image du comparateur 
     */
    public function getNameImg() {
        return $this->nameImg;
    }

    /**
     * Initialise la connexion à la base de donnée 
     */
    private function initCnx() {
        $this->cnx = StaticConnection::getHandle();
    }

    /**
     * Donne la connexion à la base de donnée
     * @return handle : connexion à la base de donnée 
     */
    public function getCnx() {
        return $this->cnx;
    }

    public function setProduitComparateur($refProd, $categorie, $old_categorie = "", $nb_clics = 0, $nb_ventes_lien = 0, $nb_ventes_produit = 0, $ca = 0) {
        if ($refProd != "" && !$this->notInArray($this->produits, $refProd, "refProd")) {
            $query = "INSERT INTO " . Shopbotexportservice::$tableCompProduit . "
                      VALUES('" . $this->idComparateur . "', '" . $refProd . "',
                          '" . mysql_real_escape_string($categorie) . "',
                          '" . mysql_real_escape_string($old_categorie) . "',
                          '" . $nb_clics . "',
                          '" . $nb_ventes_lien . "',
                          '" . $nb_ventes_produit . "',
                          '" . $ca . "');";
            mysql_query($query, $this->cnx);
            array_push($this->produits, array("refProd" => $refProd,
                "categorie" => $categorie,
                "old_categorie" => $old_categorie,
                "nb_clics" => $nb_clics,
                "nb_ventes_lien" => $nb_ventes_lien,
                "nb_ventes_produit" => $nb_ventes_produit,
                "ca" => $ca));
        } else if ($refProd != "") {
            $this->changeProduitInfo($refProd, "categorie", $categorie);
        }
        return $categorie;
    }

    /**
     * Change les informations produit
     * @param type $refProd
     * @param type $indice
     * @param type $valeur 
     */
    private function changeProduitInfo($refProd, $indice, $valeur) {
        if (count($this->produits) > 0) {
            for ($i = 0; $i < count($this->produits); $i++) {
                if ($this->produits[$i]["refProd"] == $refProd) {
                    $this->produits[$i][$indice] = $valeur;
                    $query = "UPDATE " . Shopbotexportservice::$tableCompProduit . "
                              SET $indice = '" . mysql_real_escape_string($valeur) . "'                          
                              WHERE idComp = '" . $this->idComparateur . "'
                              AND refProd = '" . $refProd . "';";
                    mysql_query($query, $this->cnx);
                }
            }
        }
    }

    /**
     * Test qu'un element, selon son indice, est présent dans le tableau
     * @param array $array : le tableau voulu
     * @param type $element : l'élement recherché
     * @param type $indice : l'indice de l'élément recherché
     * @return boolean : vrai si présent, faux sinon
     */
    private function notInArray($array, $element, $indice = "") {
        foreach ($array as $value) {
            if ($indice != "") {
                if ($value[$indice] == $element) {
                    return true;
                }
            } else {
                if ($value == $element) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Changer l'état d'un produit pour un comparateur
     * @param string $ref : la référence du produit
     * @param int $active : l'état du produit
     * @return string : les catégories à afficher pour l'état actif
     */
    public function change($ref, $active) {
        if ($active == "on") {
            $query = "UPDATE " . Shopbotexportservice::$tableCompProduit . " cp, " . Shopbotexportservice::$tableComparateur . " c
                      SET cp.old_categorie = cp.categorie,
                          categorie = ''
                      WHERE cp.idComp = c.idComp
                      AND c.idComp = '" . $this->idComparateur . "'
                      AND cp.refProd = '$ref'";
            mysql_query($query, $this->cnx);
        } else {
            $query = "UPDATE " . Shopbotexportservice::$tableCompProduit . " cp, " . Shopbotexportservice::$tableComparateur . " c
                      SET categorie = old_categorie,
                          old_categorie = ''
                      WHERE cp.idComp = c.idComp
                      AND c.idComp = '" . $this->idComparateur . "'
                      AND cp.refProd = '$ref'";
            mysql_query($query, $this->cnx);
            $query = "SELECT categorie
                      FROM " . Shopbotexportservice::$tableCompProduit . " cp, " . Shopbotexportservice::$tableComparateur . " c
                      WHERE cp.idComp = c.idComp
                      AND c.idComp = '" . $this->idComparateur . "'
                      AND cp.refProd = '$ref'";
            $ret = mysql_query($query, $this->cnx);
            if ($ret && mysql_num_rows($ret) > 0 && $row = mysql_fetch_object($ret)) {
                if($row->categorie != "") {
                    return $row->categorie;
                } else {
                    mysql_query("DELETE FROM ".Shopbotexportservice::$tableCompProduit." WHERE idComp = '".$this->getIdComparateur()."' AND refProd = '".$ref."'", $this->cnx);
                }
            }
        }
        return "";
    }

    /**
     * Méthode permettant de parser les catégories nativement (à surcharger)
     * @param string $refProduit : la référence du produit associé à la catégorie
     * @return string : le HTML (options de select) des catégories 
     */
    public function parseCategories($refProduit = "") {
        $parent1 = "";
        $parent2 = "";
        $parent3 = "";
        $outhtml = "";

        $query = "SELECT ref FROM produit WHERE ref = '" . $refProduit . "'";
        $ret = mysql_query($query, $this->cnx);
        if ($ret && mysql_num_rows($ret) > 0 && $row = mysql_fetch_object($ret)) {
            $refProduit = $row->ref;
        } else {
            $refProduit = "";
        }

        $query = "SELECT categorie FROM " . Shopbotexportservice::$tableCompProduit . " WHERE idComp = '" . $this->idComparateur . "' AND refProd = '$refProduit'";
        $ret = mysql_query($query, $this->cnx);
        if ($ret && mysql_num_rows($ret) > 0 && $row = mysql_fetch_object($ret)) {
            $cat = $row->categorie;
        } else {
            $cat = "";
        }
        if (isset($this->nom) && $this->nom != "") {
            $xml = simplexml_load_file($this->rootPath . "categories.xml");
            foreach ($xml->children() as $child) {
                $valeur = $child;
                $valeur = str_replace("\"", "", $valeur);
                $valeur = $valeur;
                $valeur = trim($valeur);
                if ($child->attributes() == "1") {
                    $outhtml .= "<option value=\"$valeur\">$valeur</option>";
                    $parent1 = $valeur;
                } elseif ($child->attributes() == "2") {
                    $outhtml .= "<option value=\"$parent1 > $valeur\">--$valeur</option>";
                    $parent2 = $valeur;
                } elseif ($child->attributes() == "3") {
                    $outhtml .= "<option value=\"$parent1 > $parent2 > $valeur\" >------$valeur</option>";
                    $parent3 = $valeur;
                } elseif ($child->attributes() == "4") {
                    $outhtml .= "<option value=\"$parent1 > $parent2 > $parent3 > $valeur\" >-----------$valeur</option>";
                }
            }
        }
        return $outhtml;
    }

    /**
     * Obtenir le chiffre d'affaire total généré pour un comparateur
     * @return <int> nb : le chiffre d'affaire généré (€)
     */
    public function getCAComparateur() {
        $query = "SELECT SUM(ca) as nb
                  FROM " . Shopbotexportservice::$tableCompProduit . "
                  WHERE idComp = " . $this->idComparateur;
        $result = mysql_query($query, $this->cnx);
        if ($result && mysql_num_rows($result) > 0 && $row = mysql_fetch_object($result) && $row->nb != "") {
            return $row->nb;
        }
        return 0;
    }
    
     /**
     * Obtenir le chiffre d'affaire total généré pour un comparateur
     * @param string ref : la référence du produit
     * @return int nb : le chiffre d'affaire généré (€)
     */
    public function getCAProduit($ref) {
        $query = "SELECT ca as nb
                  FROM " . Shopbotexportservice::$tableCompProduit . "
                  WHERE idComp = " . $this->idComparateur."
                  AND refProd = '".$ref."'";
        $result = mysql_query($query, $this->cnx);
        if ($result && mysql_num_rows($result) > 0 && $row = mysql_fetch_object($result) && $row->nb != "") {
            return $row->nb;
        }
        return 0;
    }

    /**
     * Obtenir le nombre de clics effectué avant le passage de la commande
     * @param <string> $ref : la référence du produit
     * @return <int> : le nombre de clic total
     */
    public function getNombreClics() {

        $query = "SELECT nb_clics
                  FROM " . Shopbotexportservice::$tableCompProduit . " 
                  WHERE idComp = '" . $this->idComparateur . "'";
        $ret = mysql_query($query, $this->cnx);
        if ($ret && mysql_num_rows($ret) > 0 && $row = mysql_fetch_object($ret)) {
            if ($row->nb_clics != "") {
                return $row->nb_clics;
            }
            return 0;
        }
        return 0;
    }
    
    /**
     * Obtenir le nombre de clics effectué avant le passage de la commande
     * @param <string> $ref : la référence du produit
     * @return <int> : le nombre de clic total
     */
    public function getNombreClicsProduit($ref) {

        $query = "SELECT nb_clics
                  FROM " . Shopbotexportservice::$tableCompProduit . " ci
                  WHERE ci.idComp = '" . $this->idComparateur . "'
                  AND refProd = '".$ref."'";
        $ret = mysql_query($query, $this->cnx);
        if ($ret && mysql_num_rows($ret) > 0 && $row = mysql_fetch_object($ret)) {
            if ($row->nb_clics != "") {
                return $row->nb_clics;
            }
            return 0;
        }
        return 0;
    }

    /**
     * Obtenir le nombre de jour entre le premier clic et la commande pour un produit et un comparateur
     * @param <string> $ref :  la référence du produit
     * @return <int> : le nombre de jours
     */
    public function getNombreJours($ref) {
        $query = "SELECT DATEDIFF('c.date', MIN(ci.date)) as nb
                  FROM " . Shopbotexportservice::$tableInfos . " ci, commande co, venteprod vp, " . Shopbotexportservice::$tableComparateur . " c
                  WHERE ci.ref = vp.ref
                    AND vp.commande = c.id
                    AND ci.idComp = c.idComp
                    AND c.idComp = '" . $this->idComparateur . "'
                    AND co.ref = '" . $ref . "'
                    AND ci.date <= co.date";
        $ret = mysql_query($query, $this->cnx);
        if ($ret && mysql_num_rows($ret) > 0 && $row = mysql_fetch_object($ret)) {
            if ($row->nb > 0) {
                return $row->nb;
            }
        }
        return 0;
    }

    /**
     * Donne la catégorie d'un produit selon sa référence
     * @param string $ref : la référence du produit
     * @return string : la catégorie du produit
     */
    public function getCategorie($ref) {
        $query = "SELECT categorie
                  FROM " . Shopbotexportservice::$tableCompProduit . "
                  WHERE refProd = '" . $ref . "'
                  AND idComp = '" . $this->idComparateur . "'";
        $result = mysql_query($query, $this->cnx);
        if ($result && mysql_num_rows($result) > 0 && $row = mysql_fetch_object($result)) {
            if ($row->categorie != "") {
                return $row->categorie;
            } 
        }
        return "Aucune catégorie associée.";
    }
    
    /**
     * Donne la catégorie d'un produit selon sa référence
     * @param string $ref : la référence du produit
     * @return string : la catégorie du produit
     */
    public function getOldCategorie($ref) {
        $query = "SELECT old_categorie
                  FROM " . Shopbotexportservice::$tableCompProduit . "
                  WHERE refProd = '" . $ref . "'
                  AND idComp = '" . $this->idComparateur . "'";
        $result = mysql_query($query, $this->cnx);
        if ($result && mysql_num_rows($result) > 0 && $row = mysql_fetch_object($result)) {
            if ($row->old_categorie != "") {
                return $row->old_categorie;
            } 
        }
        return "";
    }

    /**
     * Insérer des catégories pour chaque comparateur selon un produit
     * @param string $cat : la catégorie du comparateur
     * @param string $refproduit : la référence produit
     */
    public function insertion($cat, $refProduit) {
        $query = "SELECT ref FROM produit WHERE ref= '" . $refproduit . "'";
        $ret = mysql_query($query, $this->cnx);
        if ($ret && mysql_num_rows($ret) > 0 && $row = mysql_fetch_object($ret)) {
            $refProduit = $row->ref;
        } else {
            $refProduit = "";
        }

        $query = "DELETE FROM " . Shopbotexportservice::$tableCompProduit . " WHERE refProd = '$refProduit'";
        mysql_query($query, $this->cnx);
        if ($cat != "") {
            $query = "INSERT INTO " . Shopbotexportservice::$tableCompProduit . " VALUES ('" . $this->getIdComparateur() . "', '$refProduit', '" . mysql_real_escape_string($cat) . "', '', '0', '0', '0', '0')";
            mysql_query($query, $this->cnx);
        }
    }

    /**
     * Méthode permettant de générer le catalogue XML du comparateur
     * @ return XML : le catalogue XML du comparateur
     */
    public abstract function flux();
}

?>
